package FixMyStreet::Geocode::Erfurt;

use warnings;
use strict;

use Geo::Coordinates::UTM;

sub string {
	my ($cls, $s, $c) = @_;

	my $params = $c->cobrand->disambiguate_location($s);
	# Allow cobrand to fixup the user input
	$s = $params->{string} if $params->{string};

	my $search = $s;
	$s = FixMyStreet::Geocode::escape($s);

	# find suggestions by input
	my $url_suggest = "https://geoportal.erfurt.de/arcgis/rest/services/Locator/ADRESS/GeocodeServer/suggest?text=$s&maxSuggestions=5&f=pjson";
	my $js_suggest = FixMyStreet::Geocode::cache('erfurt', $url_suggest);
	if (!$js_suggest) {
		return { error => _('Sorry, we could not parse that location. Please try again.') };
	}

	my $suggestions = $js_suggest->{suggestions};

	if(scalar @$suggestions == 0) {
		return { error => _('Sorry, we could not parse that location. Please try again.') };
	}

	my ( $error, @valid_locations, $latitude, $longitude, $address, $magic_key );

	#prework for suggestions analysis
	my @search_split = split(' ', $search);

	# analyse suggestions and create result list
	foreach (@$suggestions) {
		my $input_found = 0;

		$address = $_->{text};
		$magic_key = $_->{magicKey};

		# get ETRS89 coordinates
		my $url_coord = "https://geoportal.erfurt.de/arcgis/rest/services/Locator/ADRESS/GeocodeServer/findAddressCandidates?magicKey=$magic_key&f=pjson";
		my $js_coord = FixMyStreet::Geocode::cache('erfurt', $url_coord);

		my $spatial_ref = $js_coord->{spatialReference}->{wkid};
		my $x_coord = $js_coord->{candidates}->[0]->{location}->{x};
		my $y_coord = $js_coord->{candidates}->[0]->{location}->{y};

		# calc WGS84 coordinates
		($latitude, $longitude) = Geo::Coordinates::UTM::utm_to_latlon(23, '32U', $x_coord, $y_coord);
		
		# stop when input is in suggestions list, return only match
		if(lc FixMyStreet::Geocode::escape($address) eq lc $s) {
			@valid_locations = ();
			$input_found = 1;
		}
		#stop when input street is in suggestions list, return first entry
		else {
			my @address_split = split(' ', $address);
			if(scalar @search_split == scalar @address_split - 1) {
				$input_found = 1;
				for(my $i = 0; $i < scalar @search_split; $i++) {
					if(lc $search_split[$i] ne lc $address_split[$i]) {
						$input_found = 0;
						last;
					}
				}
			}
		}
		
		push (@$error, {
			address => "$address",
			latitude => "$latitude",
			longitude => "$longitude"
			});

		push (@valid_locations, $_);

		if($input_found) {
			last;
		}
	}

	return { latitude => "$latitude", longitude => "$longitude" } if scalar @valid_locations == 1;
	return { error => $error };
}
1;
