# FixMyStreet:Map::Erfurt

package FixMyStreet::Map::Erfurt;
use base 'FixMyStreet::Map::WMTSBase';
use Geo::Coordinates::UTM;

use strict;

sub default_zoom { 6; }

sub urls { [ 'https://geoportal.erfurt.de/arcgis/rest/services/WebOffice/Stadtkarte_farbig_Internet/MapServer/WMTS/tile' ] }

sub layer_names { [ 'WebOffice_Stadtkarte_farbig_Internet' ] }

sub scales {
    my $self = shift;
    my @scales = (
        '128000',
        '64000',
        '32000',
        '16000',
        '8000',
        '4000',
        '2000',
        '1000',
    );
    return @scales;
}

sub zoom_parameters {
    my $self = shift;
    my $params = {
        zoom_levels    => scalar $self->scales,
        default_zoom   => $self->default_zoom,
        min_zoom_level => 0,
        id_offset      => 0,
    };
    return $params;
}

sub tile_parameters {
    my $self = shift;
    my $params = {
        urls            => $self->urls,
        layer_names     => $self->layer_names,
        wmts_version    => '1.0.0',
        layer_style     => 'default',
        matrix_set      => 'default028mm',
        suffix          => '.png', # appended to tile URLs
        size            => 256, # pixels
        dpi             => 96,
        inches_per_unit => 39.3701, # BNG uses metres
        projection      => 'EPSG:25832',
        origin_x        => -5120900.0,
        origin_y        => 9998100.0,
    };
    return $params;
}

sub copyright {
    return '&copy; Stadtverwaltung Erfurt';
}

sub map_javascript { [
    '/vendor/OpenLayers/OpenLayers.wmts.js',
    '/js/map-OpenLayers.js',
    '/js/map-wmts-base.js',
    '/js/map-wmts-erfurt.js',
    '/js/OpenLayers.Projection.Erfurt.js',
    '/js/proj4.js',
] }

sub map_template { 'wmts' };

# Reproject a WGS84 lat/lon into ETRS89 easting/northing
sub reproject_from_latlon($$$) {
    my ($self, $lat, $lon) = @_;
    my ($z, $x, $y) = Geo::Coordinates::UTM::latlon_to_utm(23, $lat, $lon);
    return ($x, $y);
}

# Reproject a ETRS89 easting/northing into WGS84 lat/lon
sub reproject_to_latlon($$$) {
    my ($self, $x, $y) = @_;
    my ($lat, $lon) = Geo::Coordinates::UTM::utm_to_latlon(23, '32U', $x, $y);
    return ($lat, $lon);
}

1;
