/**
 * OpenLayers Erfurt grid projection transformations
 * 
 * Provides transform functions for WGS84<->ETRS89 projections by using PROJ4JS.
 * http://proj4js.org
 */

OpenLayers.Projection.Erfurt = {
    // Function to convert a WGS84 coordinate to a ETRS89 coordinate.
    projectForwardErfurt: function(point) {
	// define projection for ETRS89
	proj4.defs("EPSG:25832","+proj=utm +zone=32 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
	
	// do the conversion
	var source = new proj4.Proj('EPSG:4326');
	var dest = new proj4.Proj('EPSG:25832');
	var result = proj4(source, dest, point);

	// return new coordinates
	point.x = result.x;
	point.y = result.y;
	return point;
    },

    // Function to convert a ETRS89 coordinate to a WGS84 coordinate. 
    projectInverseErfurt: function(point) {
	// define projection for ETRS89
	proj4.defs("EPSG:25832","+proj=utm +zone=32 +ellps=GRS80 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs");
	
	// do the conversion
	var source = new proj4.Proj('EPSG:25832');
	var dest = new proj4.Proj('EPSG:4326');
	var result = proj4(source, dest, point);
	
	// return new coordinates
	point.x = result.x;
	point.y = result.y;
	return point;
    }
};

/**
 * Note: One transform declared
 * Transforms from EPSG:4326 to EPSG:25832
 */
OpenLayers.Projection.addTransform("EPSG:4326", "EPSG:25832",
   OpenLayers.Projection.Erfurt.projectForwardErfurt);
OpenLayers.Projection.addTransform("EPSG:25832", "EPSG:4326",
   OpenLayers.Projection.Erfurt.projectInverseErfurt);
