use utf8;
package FixMyStreet::DB::Result::ResponseTemplate;

# Created by DBIx::Class::Schema::Loader
# DO NOT MODIFY THE FIRST PART OF THIS FILE

use strict;
use warnings;

use base 'DBIx::Class::Core';
__PACKAGE__->load_components(
  "FilterColumn",
  "FixMyStreet::InflateColumn::DateTime",
  "FixMyStreet::EncodedColumn",
);
__PACKAGE__->table("response_templates");
__PACKAGE__->add_columns(
  "id",
  {
    data_type         => "integer",
    is_auto_increment => 1,
    is_nullable       => 0,
    sequence          => "response_templates_id_seq",
  },
  "body_id",
  { data_type => "integer", is_foreign_key => 1, is_nullable => 0 },
  "title",
  { data_type => "text", is_nullable => 0 },
  "text",
  { data_type => "text", is_nullable => 0 },
  "created",
  {
    data_type     => "timestamp",
    default_value => \"current_timestamp",
    is_nullable   => 0,
    original      => { default_value => \"now()" },
  },
  "auto_response",
  { data_type => "boolean", default_value => \"false", is_nullable => 0 },
  "state",
  { data_type => "text", is_nullable => 1 },
  "external_status_code",
  { data_type => "text", is_nullable => 1 },
  "email_text",
  { data_type => "text", is_nullable => 1 },
);
__PACKAGE__->set_primary_key("id");
__PACKAGE__->add_unique_constraint("response_templates_body_id_title_key", ["body_id", "title"]);
__PACKAGE__->belongs_to(
  "body",
  "FixMyStreet::DB::Result::Body",
  { id => "body_id" },
  { is_deferrable => 0, on_delete => "NO ACTION", on_update => "NO ACTION" },
);
__PACKAGE__->has_many(
  "contact_response_templates",
  "FixMyStreet::DB::Result::ContactResponseTemplate",
  { "foreign.response_template_id" => "self.id" },
  { cascade_copy => 0, cascade_delete => 0 },
);


# Created by DBIx::Class::Schema::Loader v0.07035 @ 2019-04-25 12:06:39
# DO NOT MODIFY THIS OR ANYTHING ABOVE! md5sum:MzTa7p2rryKkxbRi7zN+Uw

__PACKAGE__->many_to_many( contacts => 'contact_response_templates', 'contact' );

use Utils;

# Trim whitespace for external_status_code

sub new {
    my ( $self, $attrs ) = @_;

    $attrs->{external_status_code}
        = Utils::trim_text( $attrs->{external_status_code} )
        if $attrs->{external_status_code};

    return $self->next::method($attrs);
}

sub set_column {
    my ( $self, $column, $new_value ) = @_;

    if ( $column eq 'external_status_code' ) {
        $new_value = Utils::trim_text($new_value);
    }

    $self->next::method( $column, $new_value );
};

1;
